'***************************************************************
' Description:  Populate Users Comment Attribute in AD with last computer logged into
'		Places the computer name in the comment field
' 		Need to ensure the Users Object in AD has self permissions set on the Comment Attribute
'
' Concept:  Dan Holme
' Original Author: Rob Dunn
' Update by: Gary Long / Chris Meyers
'
' Version: 1.0
'
' Date:  03/10/2015
'
'***************************************************************

ON ERROR GOTO 0

DIM strUserDN
DIM iMaxComputersSaved

'***************************************************************
'How many computernames do you want to save in the comment attribute.
'***************************************************************
iMaxComputersSaved = 5

'***************************************************************
'Delimiter used to separate the entries
'***************************************************************
strDelim = "|" 

SET WshShell = WScript.CreateObject("WScript.Shell")
SET WshSysEnv = WshShell.Environment("PROCESS")

'***************************************************************
'Get the user, computer, and domain name from the local machine
'***************************************************************
strUser = WshSysEnv("username")
strComputer = wshShell.ExpandEnvironmentStrings("%Computername%")

SET objSystemInfo = CreateObject("ADSystemInfo")
strDomain = objSystemInfo.DomainShortName

'***************************************************************
'Place the DN into a variable and bind to it in AD
'***************************************************************
strUserDN = GetUserDN(strUser,strDomain)
SET objUser = GetObject ("LDAP://" & struserDN)

'***************************************************************
'Read the current comment and place in a variable
'***************************************************************
strComment = objUser.comment

'Split the current comment into an array and evaluate it
tmpArray = split(objUser.comment,"|")

DIM d   ' Create a variable.
SET d = CreateObject("Scripting.Dictionary")

on error resume next
For i = 0 to ubound(tmpArray)
  'create a temporary array by splitting the item at the '-' delimiter.
  ' this separates the computer name with the date/time that was entered.
  tmpArray2 = split(tmpArray(i)," - ")
  IF trim(lcase(tmpArray(i))) = lcase(strComputer) then 
     'if the current computer name matches the current array item, then...
     ' ...don't do anything.     
  Else
    'if the current computer name does not match the current array item, then
    ' add the current array item to the dictionary
    d.Add tmpArray2(0), tmpArray2(1)
  END IF
NEXT

d.Add strComputer, Now   

'***************************************************************
'clear the comment field and commit to AD
'***************************************************************
objUser.PutEx 1, "comment", 0
objUser.SetInfo

IF d.count >= iMaxComputersSaved then 
  n = d.count - iMaxComputersSaved
  'i.e. we have 15 items in d.count and 10 items in computers to save, then
  ' n (or offset) would = 5.  This eventually helps us keep the last x computer
  ' names in the list, rather than the first x computer names.
Else
  n = 0
END IF

c = 0

'iterate through each item in the dictionary.
For each objItem in d
  
  'if our count is greater or equal to the offset between maxcomputers and 
  ' the count of existing items in the comment field, then go ahead and 
  ' append the info to the comment text.  I.e. if we have 15 items in our 
  ' dictionary and want to only keep 10 items in our history, then this allows
  ' us to drop the first five computer names.
  IF c >= n then
      'append the computername and date/time stamp to the comment string 
      ' variable.  
      strComment1 = strComment1 & delim & objItem & " - " & d.item(objitem) 
      delim = strDelim 
  END IF
  c = c + 1
NEXT 

'***************************************************************
'Write the new info to the comment field and committ to AD
'***************************************************************
objUser.put "comment", strComment1
objUser.SetInfo

'***********************************************************
FUNCTION GetUserDN(byval strUserName,byval strDomain)
  'http://www.wisesoft.co.uk/Scripts/display_script.aspx?id=118
  ' ^- original source of function...
  ' Use name translate to return the distinguished name
  ' of a user from the NT UserName (sAMAccountName)
  ' and the NETBIOS domain name.
  SET objTrans = CreateObject("NameTranslate")
  objTrans.Init 1, strDomain
  objTrans.Set 3, strDomain & "\" & strUserName
  strUserDN = objTrans.Get(1)
  GetUserDN = strUserDN
END FUNCTION
